@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: nc.c,v 2.135 2004/09/30 18:47:12 russ Exp $ */
+/* $Id: nc.c,v 2.143 2006/01/28 20:01:19 ed Exp $ */
 
 #include "nc.h"
 #include "rnd.h"
@@ -19,7 +19,7 @@
 static NC *NClist = NULL;
 
 /* This is the default create format for nc_create and nc__create. */
-static int default_create_format = NC_FORMAT_CLASSIC;
+int default_create_format = NC_FORMAT_CLASSIC;
 
 /* These have to do with version numbers. */
 #define MAGIC_NUM_LEN 4
@@ -849,6 +849,51 @@ NC_init_pe(NC *ncp, int basepe) {
 }
 #endif
 
+
+/*
+ * Compute the expected size of the file.
+ */
+int
+NC_calcsize(NC *ncp, off_t *calcsizep)
+{
+	NC_var **vpp = (NC_var **)ncp->vars.value;
+	NC_var *const *const end = &vpp[ncp->vars.nelems];
+	NC_var *last_fix = NULL;	/* last "non-record" var */
+	NC_var *last_rec = NULL;	/* last "record" var */
+	/*NC_var *last_var;*/
+	int status;
+	int numrecvars = 0;	/* number of record variables */
+
+	if(ncp->vars.nelems == 0) { /* no non-record variables and 
+				       no record variables */
+	    *calcsizep = ncp->xsz; /* size of header */
+	    return NC_NOERR;
+	}
+
+	for( /*NADA*/; vpp < end; vpp++) {
+	    status = NC_var_shape(*vpp, &ncp->dims);
+	    if(status != NC_NOERR)
+		return status;
+	    if(IS_RECVAR(*vpp)) {
+		last_rec = *vpp;
+		numrecvars++;
+	    } else {
+		last_fix = *vpp;
+	    }
+	}
+
+	if(numrecvars == 0) {
+	    assert(last_fix != NULL);
+	    *calcsizep = last_fix->begin + last_fix->len;
+	    /*last_var = last_fix;*/
+	} else {       /* we have at least one record variable */
+	    *calcsizep = ncp->begin_rec + ncp->numrecs * ncp->recsize;
+	    /*last_var = last_rec;*/
+	}
+
+	return NC_NOERR;
+}
+
 /* Public */
 
 int
@@ -962,8 +1007,14 @@ nc_set_default_format(int format, int *old_formatp)
       *old_formatp = default_create_format;
 
     /* Make sure only valid format is set. */
+#ifdef USE_NETCDF4
+    if (format != NC_FORMAT_CLASSIC && format != NC_FORMAT_64BIT &&
+	format != NC_FORMAT_NETCDF4 && format != NC_FORMAT_NETCDF4_CLASSIC)
+      return NC_EINVAL;
+#else
     if (format != NC_FORMAT_CLASSIC && format != NC_FORMAT_64BIT)
       return NC_EINVAL;
+#endif
     default_create_format = format;
     return NC_NOERR;
 }
@@ -1116,6 +1167,28 @@ nc_close(int ncid)
 		status = NC_sync(ncp);
 	}
 
+	/* 
+	 * If file opened for writing and file size is different from
+	 * what it should be (due to previous use of NOFILL mode),
+	 * change it to correct size, as reported by NC_calcsize().
+	 */
+	{
+	    off_t filesize; 	/* current size of open file */
+	    off_t calcsize;	/* calculated file size, from header */
+	    status = ncio_filesize(ncp->nciop, &filesize);
+	    if(status != ENOERR)
+		return status;
+	    status = NC_calcsize(ncp, &calcsize);
+	    if(status != NC_NOERR)
+		return status;
+	    if(filesize < calcsize && !NC_readonly(ncp)) {
+		status = ncio_pad_length(ncp->nciop, calcsize);
+		if(status != ENOERR)
+		    return status;
+	    }
+	    
+	}
+
 	(void) ncio_close(ncp->nciop, 0);
 	ncp->nciop = NULL;
 
@@ -1518,3 +1591,20 @@ nc_inq_base_pe(int ncid, int *pe)
 #endif /* _CRAYMPP && LOCKNUMREC */
 	return NC_NOERR;
 }
+
+int
+nc_inq_format(int ncid, int *formatp)
+{
+	int status;
+	NC *ncp;
+
+	status = NC_check_id(ncid, &ncp); 
+	if(status != NC_NOERR)
+		return status;
+
+	/* only need to check for netCDF-3 variants, since this is never called for netCDF-4 
+	   files */
+	*formatp = fIsSet(ncp->flags, NC_64BIT_OFFSET) ? NC_FORMAT_64BIT 
+	    : NC_FORMAT_CLASSIC; 
+	return NC_NOERR;
+}
